/*
Unnamed DNS Server (http://eder.us/projects/unnamed/
Copyright (C) 2012 David Eder

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "unnamed.h"

typedef struct
{
  char *version_string;
  char *hostname;
  uint32_t ttl;
  int trace;
} module_data_t;

static void term(int module_index, state_t *state)
{
  module_data_t *module = (module_data_t*)state->module_data[module_index];
  free((void*)module->version_string);
  free((void*)module->hostname);
  state->config_free_node(state, "version");
  free(module);
  state->module_data[module_index] = NULL;
}

static int init(int module_index, state_t *state)
{
  module_data_t *module = (module_data_t*)calloc(1, sizeof(*module));
  if(module == NULL) return -1;
  state->module_data[module_index] = module;

  module->trace = state->config_get_int(state, "version.trace", TRACE_LEVEL_ERROR);
  module->ttl = (uint32_t)state->config_get_int(state, "version.ttl", DEFAULT_TTL);
  module->version_string = state->config_get(state, "version.banner");
  if(module->version_string == NULL)
  {
    ERROR_IF(module->trace) (void)state->lprintf(TRACE, "Error, version.banner is not set");
    term(module_index, state);
    return -1;
  }
  module->hostname = state->config_get(state, "hostname");
  if(module->hostname == NULL)
  {
    ERROR_IF(module->trace) (void)state->lprintf(TRACE, "Error, version.hostname is not set");
    term(module_index, state);
    return -1;
  }
  state->config_free_node(state, "version");
  return 0;
}

static answer_t answer(int module_index, client_t *client, rr_section_t rr_section, const buffer_t *name, uint16_t type, uint16_t dclass, int __attribute__((unused)) lazy)
{
  state_t *state = client->state;
  module_data_t *module = (module_data_t*)state->module_data[module_index];
  if((type == TYPE_TXT || type == TYPE_ANY) && (dclass == CLASS_CH || dclass == CLASS_ANY))
  {
    if(module->version_string != NULL && strcmp((char*)name->value, "version.bind") == 0)
    {
      rr_t *rr = state->rr_create(name, TYPE_TXT, CLASS_CH, (uint32_t)time(NULL) + module->ttl, RCODE_NO_ERROR);
      if(rr == NULL) return ANSWER_UNSATISFIED;
      if(state->buffer_set(&rr->p.txt, "%s", module->version_string) == 0)
      {
        if(state->add_rr(client, rr_section, &rr) == 0)
        {
          state->free_rr(&rr);
          INFO_IF(module->trace) (void)state->lprintf(TRACE, "%s %s %s => %s", (char*)name->value, state->class_name(dclass), state->type_name(type), module->version_string);
          return ANSWER_SATISFIED;
        }
      }
      state->free_rr(&rr);
    }
    else if(module->hostname != NULL && (strcmp((const char*)name->value, "hostname.bind") == 0 || strcmp((const char*)name->value, "id.server") == 0))
    {
      rr_t *rr = state->rr_create(name, TYPE_TXT, CLASS_CH, (uint32_t)time(NULL) + module->ttl, RCODE_NO_ERROR);
      if(rr == NULL) return ANSWER_UNSATISFIED;
      if(state->buffer_set(&rr->p.txt, "%s", module->hostname) == 0)
      {
        if(state->add_rr(client, rr_section, &rr) == 0)
        {
          state->free_rr(&rr);
          INFO_IF(module->trace) (void)state->lprintf(TRACE, "%s %s %s => %s", (char*)name->value, state->class_name(dclass), state->type_name(type), module->hostname);
          return ANSWER_SATISFIED;
        }
      }
      state->free_rr(&rr);
    }
  }
  return ANSWER_UNSATISFIED;
}

module_t unnamed_module = { "version", NULL, init, answer, NULL, term };
